

import java.util.LinkedList;

class VerticeAdjacente extends Vertice {

    double peso;

    public VerticeAdjacente(int id, double peso) {
        super(id);

        this.peso = peso;
    }

    public VerticeAdjacente(Vertice v, double peso) {
        super(v.id);
        this.adjacentes = v.adjacentes;

        this.peso = peso;
    }
}

class Vertice {

    int id;
    // lista ligada
    LinkedList<VerticeAdjacente> adjacentes;

    public Vertice(int id) {
        this.id = id;
        adjacentes = new LinkedList<VerticeAdjacente>();
    }

    // comparacao de um vertice com o vertice atual
    boolean igualA(Vertice v) {
        if (v.id == this.id) 
            return(true);
        else 
            return(false);
    }

    void insereAresta(Vertice destino, double peso) {
    
        VerticeAdjacente vA = new VerticeAdjacente(destino, peso);
        adjacentes.add(vA);
    }

    boolean estaEmAdjacentes(Vertice v) {

        for (VerticeAdjacente va: adjacentes) {
            if (va.igualA(v)) 
                return(true);
        }

        return(false);
    }

    public String toString() {

        return(""+id);
    }

    void imprimir() {
        for (VerticeAdjacente vA: adjacentes) {
            System.out.println("("+id+", "+ vA.id+") = " + vA.peso);
        }
    }

}

public class Grafo {

    LinkedList<Vertice> vertices;

    public Grafo() {
        vertices = new LinkedList<Vertice>();
    }

    // aresta tem origem e destino 
    void insereAresta(Vertice origem, Vertice destino, double peso) {

        for (Vertice vert: vertices) {
            if (vert.igualA(origem)) {
               vert.insereAresta(destino, peso); 
               return;
            }
        }

        vertices.add(origem);
        origem.insereAresta(destino, peso);

    }

    boolean existeAresta(Vertice origem, Vertice destino) {

        for (Vertice v: vertices) {

            if (v.igualA(origem)) {
                
                if (v.estaEmAdjacentes(destino)) 
                    return(true);
                else 
                    return(false);
            }
        }

        return(false);

    }

    void imprimir() {

        for (Vertice vert: vertices ) {
            vert.imprimir();
        }
    }

    Vertice[] listaAdjacentes(Vertice v) {

        for (Vertice vert: vertices) {
            if (vert.igualA(v)) {
               return(vert.adjacentes.toArray(new Vertice[0]));
            }
        }

        return(null);
    }

    Grafo transposto() {

        Grafo gT = new Grafo();

        for (Vertice v: vertices) 
            for (VerticeAdjacente va: v.adjacentes) 
                gT.insereAresta(va, v, va.peso);


        return(gT);
    }

    Vertice[] arestaMinima() {

        Vertice[] aresta = new Vertice[2];
        double pesoMinimo = Double.MAX_VALUE;

        for (Vertice origem: vertices) 
            for (VerticeAdjacente destino: origem.adjacentes) {
                
                if (destino.peso < pesoMinimo) {
                    pesoMinimo = destino.peso;
                    aresta[0] = origem;
                    aresta[1] = destino;
                }
            }

        return(aresta);
    }

    void removerAresta(Vertice origem, Vertice destino) {
        for(Vertice v: vertices) 
            for (VerticeAdjacente va: v.adjacentes) {

                if (v.igualA(origem) && va.igualA(destino)) {
                    v.adjacentes.remove(va);

                    if (v.adjacentes.size() == 0)
                        vertices.remove(v);
                }
            }
    }

    public static void main(String args[]) {
        Grafo g = new Grafo();

        g.insereAresta(new Vertice(1), new Vertice(1), 0.1);
        g.insereAresta(new Vertice(1), new Vertice(2), 0.6);
        g.insereAresta(new Vertice(2), new Vertice(1), 0.4);
        g.insereAresta(new Vertice(2), new Vertice(2), 0.4);
        g.insereAresta(new Vertice(2), new Vertice(3), 0.1);
        g.insereAresta(new Vertice(3), new Vertice(2), 0.8);

        g.imprimir();

        System.out.println("Existe (3,1) = "+   
                g.existeAresta(new Vertice(3), new Vertice(1)));

        System.out.println("Existe (4,1) = "+   
                g.existeAresta(new Vertice(4), new Vertice(1)));
        System.out.println("Existe (1,1) = "+   
                g.existeAresta(new Vertice(1), new Vertice(1)));

        Vertice[] adj2 = g.listaAdjacentes(new Vertice(2));

        for (Vertice v: adj2) 
            System.out.println(" adjacente do 2 o vrtice: "+ v.id);

        Grafo gT = g.transposto();
        gT.imprimir();

        Vertice arestaMinima[] = gT.arestaMinima();

        System.out.println("Aresta minima e gT = "+arestaMinima[0]+" "+arestaMinima[1]);



        g.removerAresta(new Vertice(2), new Vertice(3));
        g.imprimir();

    }
}
